@@ -4,13 +4,17 @@
 #include <gtk/gtkstyle.h>
 #include <libosso.h>
 #include "mpuzzle.h"
+#include <hildon/hildon-banner.h>
+#include <hildon/hildon-program.h>
+#include <sys/time.h>
+#include <time.h>
+
 
-static max_undos_supported_count=0;
-static int no_of_valid_undos=0;
-GtkWidget *undo_button;
 /*
  * Deletes the last node in the stack for restricting number of undo 
  */
+
+
 void free_last_element_in_undo_stack( struct pos **head )
 {
 	struct pos *ptrCurrent = *head;
@@ -48,12 +52,12 @@ int main (int argc, char *argv[])
 	osso_context_t* ctx = NULL;
 	gtk_init (&argc, &argv);
 	main_window = create_main_window ();
-	ctx = osso_initialize("org.maemo.mpuzzle", "1.6", TRUE, NULL);
+	ctx = osso_initialize("org.maemo.mpuzzle", "1.7.T18", TRUE, NULL);
 	if (ctx == NULL) {
-		g_print("Failed to init LibOSSO\n");
+	/*	g_print("Failed to init LibOSSO\n");*/
 		return 0;
 	}
-	g_print("LibOSSO Init done\n");
+/*	g_print("LibOSSO Init done\n");*/
 				
 	gtk_widget_show_all (main_window);
 	gtk_main ();
@@ -87,7 +91,7 @@ gboolean is_legal_slide (gint current_pos, gint vacant_pos)
 gboolean has_player_won (void)
 {
 	int x;
-	for (x=0; x<15; x++)	
+	for (x=0; x<NOOFBUTTONS; x++)	
 		if (x!= (gint) gtk_object_get_data(GTK_OBJECT(buttons[x]),"current_pos"))
 	 	    return FALSE;
 	
@@ -95,32 +99,6 @@ gboolean has_player_won (void)
 	return TRUE;
 }
 
-/* 
- * check the generated puzzle is solvable or not if not generate the random puzzle again
- */
-gboolean store_sequence_check_is_puzzle_solvable(void)
-{
-	int y;
-	int puzzle_seq[15];
-	int sum=0,i,j;
-	int blank_space_row_no;
-	for(y=0;y<15;y++){
-		puzzle_seq[y] = (gint) gtk_object_get_data(GTK_OBJECT(buttons[y]), "current_pos");
-	}
-	blank_space_row_no= (int)(vacant_pos/4)+1;
-	for(i=0;i<14;i++) {
-           for(j=i+1;j<=14;j++) {
-	        if(puzzle_seq[i]>puzzle_seq[j])	{
-	  	      sum += 1;  
-		}
-           }     
-	}
-       	if((sum + blank_space_row_no)%2 == 0){
-		return TRUE;
-	}	
-      return FALSE;
-}
-
 
  /* 
   * Action taken after button is clicked
@@ -132,9 +110,8 @@ void on_button_clicked (GtkWidget *button, gpointer user_data)
 	int x_pos, y_pos;
 	int current_pos;	
 	gchar file_name[MAXSIZE];	
-	struct pos *temp_undo_store_positions;
-		
-	temp_undo_store_positions = (struct pos *)(malloc(sizeof(struct pos)));
+	
+	struct pos *temp_undo_store_positions = (struct pos *)(malloc(sizeof(struct pos)));
 	current_pos = (gint) gtk_object_get_data (GTK_OBJECT(button), "current_pos");
 
 			
@@ -142,7 +119,7 @@ void on_button_clicked (GtkWidget *button, gpointer user_data)
        		gtk_widget_set_sensitive(undo_button,TRUE); /* Activate undo button*/
 		max_undos_supported_count++;	
 			
-		x_pos = vacant_pos%4; y_pos = (int)(vacant_pos/4);
+		x_pos = vacant_pos%CELLPOS; y_pos = (int)(vacant_pos/CELLPOS);
 		
 		 if(temp_undo_store_positions){    
 		     temp_undo_store_positions->attach_in_vac_pos = vacant_pos;
@@ -151,7 +128,6 @@ void on_button_clicked (GtkWidget *button, gpointer user_data)
 		 }else{
 			 g_print("unable to allocate memory from heap for undo strcut");			 
 			 gtk_main_quit();
-			/* Quit application */
 		 }
 
 		no_of_valid_undos++;		
@@ -195,7 +171,6 @@ GtkWidget* create_main_window (void)
   GtkWidget *separatormenuitem1,*quit1,*menuitem4;
   GtkWidget *menuitem4_menu,*about1,*toolbar1;
   GtkWidget *new_button,*quit_button,*help_button;
-/*  GtkWidget *undo_button;*/
   
   gint tmp_toolbar_icon_size, x, y, c=0;
   GtkAccelGroup *accel_group = gtk_accel_group_new ();
@@ -204,7 +179,7 @@ GtkWidget* create_main_window (void)
   vacant_pos = 15; move_no=0;
 
   main_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-  gtk_window_set_title (GTK_WINDOW (main_window), "MPuzzle-1.6");
+  gtk_window_set_title (GTK_WINDOW (main_window), "MPuzzle-1.7.T18");
   vbox1 = gtk_vbox_new (FALSE, 0);
   gtk_container_add (GTK_CONTAINER (main_window), vbox1);
 
@@ -250,13 +225,15 @@ GtkWidget* create_main_window (void)
   gtk_box_pack_start (GTK_BOX (vbox1), alignment1, TRUE, TRUE, 0);
   gtk_alignment_set_padding (GTK_ALIGNMENT (alignment1), 20, 20, 40, 40);
 
+  
   table1 = gtk_table_new (4, 4, TRUE);
   gtk_container_add (GTK_CONTAINER (alignment1), table1);
   gtk_widget_modify_bg(main_window, GTK_STATE_NORMAL, &color);
+  
 
   gtk_widget_set_sensitive(undo_button,FALSE);
   
-  for (x=0; x<15; x++) {
+  for (x=0; x<NOOFBUTTONS; x++) {
 	  gchar *temp = g_strdup_printf ("%i", x+1);
 	  buttons[x] = gtk_button_new_with_mnemonic (temp);
 	  gtk_object_set_data (GTK_OBJECT(buttons[x]), "current_pos", (gpointer) x);
@@ -271,8 +248,8 @@ GtkWidget* create_main_window (void)
 	  g_free (temp);
   }
  
-  for (x=0; x<4; x++)
-  	  for (y=0; y<4; y++)
+  for (x=0; x<CELLPOS; x++)
+  	  for (y=0; y<CELLPOS; y++)
   		  if (!(x==3 && y==3))
 			  gtk_table_attach (GTK_TABLE (table1), 
 	  		  buttons[c++], y,y+1,x,x+1,
@@ -280,7 +257,7 @@ GtkWidget* create_main_window (void)
       			  (GtkAttachOptions) GTK_FILL | GTK_EXPAND,
       			  (GtkAttachOptions) 1, 1);
   
-  for (x=0; x<15; x++)
+  for (x=0; x<NOOFBUTTONS; x++)
 	gtk_signal_connect(GTK_OBJECT(buttons[x]), "clicked", GTK_SIGNAL_FUNC(on_button_clicked), (gpointer)x);
 
   statusbar1 = gtk_statusbar_new ();
@@ -298,98 +275,181 @@ GtkWidget* create_main_window (void)
   g_signal_connect (GTK_OBJECT(undo_button),"clicked", GTK_SIGNAL_FUNC(on_undo_activate),NULL);
 	
   gtk_window_add_accel_group (GTK_WINDOW (main_window), accel_group);
-  gtk_widget_grab_focus(new_button);
-  gtk_button_clicked (new_button);
-
-  gtk_widget_grab_focus(undo_button);
-  gtk_button_clicked (undo_button);
-
 
   return main_window;
   
 }
 
-void swap_buttons (int p, int q)
-{
-	gint p_pos = (gint) gtk_object_get_data (GTK_OBJECT(buttons[p]), "current_pos");
-	gint q_pos = (gint) gtk_object_get_data (GTK_OBJECT(buttons[q]), "current_pos");
-	int p_x_pos = p_pos%4; 
-	int p_y_pos = (int)(p_pos/4);
-	int q_x_pos = q_pos%4; 
-	int q_y_pos = (int)(q_pos/4);
-
-	gtk_object_ref (GTK_OBJECT(buttons[p]));
-	gtk_object_ref (GTK_OBJECT(buttons[q]));
-	gtk_container_remove (GTK_CONTAINER(table1), buttons[p]);
-	gtk_container_remove (GTK_CONTAINER(table1), buttons[q]);
-
-	gtk_table_attach (GTK_TABLE (table1), 
-			buttons[p], q_x_pos, q_x_pos + 1, q_y_pos, q_y_pos + 1,
-			(GtkAttachOptions) GTK_FILL | GTK_EXPAND,
-			(GtkAttachOptions) GTK_FILL | GTK_EXPAND,
-			(GtkAttachOptions) 1, 1);
-	gtk_table_attach (GTK_TABLE (table1), 
-			buttons[q], p_x_pos, p_x_pos + 1, p_y_pos, p_y_pos + 1,
-			(GtkAttachOptions) GTK_FILL | GTK_EXPAND,
-			(GtkAttachOptions) GTK_FILL | GTK_EXPAND,
-			(GtkAttachOptions) 1, 1);
-    	gtk_object_set_data (GTK_OBJECT(buttons[p]), "current_pos", (gpointer) q_pos);
-	gtk_object_set_data (GTK_OBJECT(buttons[q]), "current_pos", (gpointer) p_pos);
-}
-
 
 void on_new_activate (GtkWidget *button,GtkMenuItem *menuitem, gpointer user_data)
 {
-	generate_puzzle();	
-}
+	int i;
+	int y,x,k,rand_pos;
+	valid_moves_count=0;
+
+	vacant_prev_pos = vacant_pos;
+	vacant_pos=15;
 
-void generate_puzzle()
-{
-	int rand,x;
 	DeleteAllElements_in_undo_stack(&undo_store_positions);
 
 	gtk_statusbar_push (GTK_STATUSBAR(statusbar1), 0, "Welcome to MARS Telecom Puzzle");
 	move_no=0;
 
-        if (GTK_IS_WIDGET(image) && GTK_IS_WIDGET(image->parent)) {
-               gtk_object_ref (GTK_OBJECT(image));
-               gtk_container_remove (GTK_CONTAINER(alignment1), image); 
-               gtk_container_add (GTK_CONTAINER(alignment1), table1);
-               gtk_widget_show (table1);
-        }
-        for (x=0; x<=14; x++) {
-               rand = abs(g_rand_int ( g_rand_new () )) % 15;
-               if (x!=rand) 
-                swap_buttons (x, rand);
-       }
-       if(store_sequence_check_is_puzzle_solvable()) {
-	   g_print("\n solvable puzzle");
-           return;
-       } else{   
-             g_print("\n Generating Puzzle Again\n");
-	     DeleteAllElements_in_undo_stack(&undo_store_positions);
-             generate_puzzle();
-       }
+
+	if (GTK_IS_WIDGET(image) && GTK_IS_WIDGET(image->parent)) {
+		gtk_object_ref (GTK_OBJECT(image));
+		gtk_container_remove (GTK_CONTAINER(alignment1), image); 
+		gtk_container_add (GTK_CONTAINER(alignment1), table1);
+		gtk_widget_show (table1);
+	}
+	
+
+	for(x=0;x<NOOFBUTTONS;x++)
+		alist[x]=x;
+	
+	gint row_pos,col_pos,req_pos;
+ 	gboolean ismoveValid;
+
+
+	for(i=0;i<NOOFMOVES;i++){
+	 
+/*		movpos = abs(g_rand_int ( g_rand_new () )) % CELLPOS;	*/
+		row_pos = (int) vacant_pos/CELLPOS;
+		col_pos = vacant_pos%CELLPOS;	
+		ismoveValid=FALSE;
+
+		valid_move_positions(vacant_pos);
+		
+		rand_pos= abs(g_rand_int ( g_rand_new () ) ) % valid_moves_count ;
+				
+		switch(val_pos[rand_pos])
+		{
+			case UP:
+				if(row_pos != ZERO){
+					req_pos = (((row_pos-1)*CELLPOS) + col_pos);
+					ismoveValid = TRUE;
+				}
+				break;			       
+			case DOWN:
+				if(row_pos != THREE){
+					req_pos = (((row_pos + 1)* CELLPOS) + col_pos);
+					ismoveValid=TRUE;
+				}
+				break;
+			case LEFT:
+				if(col_pos != ZERO){
+					req_pos = (row_pos*CELLPOS) + (col_pos-1);
+					ismoveValid = TRUE;
+				}
+				break;
+			case RIGHT:
+				if(col_pos != THREE){
+					req_pos = (row_pos*CELLPOS) + (col_pos+1);
+					ismoveValid = TRUE;
+				}
+				break;
+		}
+	
+	
+	
+	if(ismoveValid){
+		for(k=0;k<NOOFBUTTONS;k++)
+				if(alist[k] == req_pos)
+					break;					
+		
+			
+		alist[k]=vacant_pos;
+		vacant_pos = req_pos;
+		
+	}
+}
+	
+	for(i=0;i<NOOFBUTTONS;i++){
+		gtk_object_ref (GTK_OBJECT(buttons[i]));
+		gtk_container_remove (GTK_CONTAINER(table1),buttons[i] );
+		y=alist[i]%CELLPOS;
+		x= (int) alist[i]/CELLPOS;	
+		gtk_table_attach (GTK_TABLE (table1), 
+				buttons[i], y,y+1,x,x+1,
+				(GtkAttachOptions) GTK_FILL | GTK_EXPAND,
+				(GtkAttachOptions) GTK_FILL | GTK_EXPAND,
+				(GtkAttachOptions) 1, 1);
+		gtk_object_set_data (GTK_OBJECT(buttons[i]), "current_pos", (gpointer) alist[i]);
+	}	
+
 	
 }
 
+		
+/* Below Code to optimize the selections of slide position either UP,DOWN,LEFT, RIGHT 
+ * selecting Random operation, only in the Valid moves                                  
+ * argument vcpos -> vacant position
+ * global variable val_pos[4]=> Stores valid move positions accordingly
+ * globa variable valid_moves_count => counts max number of valid moves which will be used in on_new_activate () function   */
 
-/* 
+		
+void valid_move_positions(gint vcpos) {
+	int a= -1,b= -1,c= -1,d = -1;
+	int i=0;
+	valid_moves_count=0;
+	gint row_pos,col_pos;
+	col_pos = vcpos%4;
+	row_pos = (int) vcpos/4;
+
+	if((row_pos-1)<=3 && (row_pos-1)>=0){
+		b = 4*(row_pos-1)+col_pos;			
+		if(b!=-1){
+			val_pos[i]=UP;
+			i++;
+			valid_moves_count++;
+		}
+	}
+	if((row_pos+1)<=3 && (row_pos+1)>=0){
+		a = 4*(row_pos+1)+col_pos;
+		if(a!=-1){
+			val_pos[i]=DOWN;
+			i++;
+			valid_moves_count++;
+		}
+	}
+	if((col_pos-1)<=3 && (col_pos-1)>=0){
+		d = 4*row_pos+(col_pos-1);
+		if(d!=-1){
+			val_pos[i]=LEFT;
+			i++;
+			valid_moves_count++;
+		}
+	}
+
+	if((col_pos+1)<=3 && (col_pos+1)>=0){
+		c = 4*row_pos+(col_pos+1);
+		if(c!=-1){
+			val_pos[i]=RIGHT;
+			i++;
+			valid_moves_count++;
+
+		}
+	}
+
+}
+		
+
+/*
  * Action taken when undo button is pressed
  */ 
+
 void on_undo_activate(GtkWidget *button,gpointer user_data)
 {
+	int k=0;
+	gchar file_name[MAXSIZE];
 	if(max_undos_supported_count>0) {
 		max_undos_supported_count--;
 	}
-	gchar file_name[MAXSIZE];
-	struct pos *temp_undo_store_positions;
 	gtk_statusbar_push (GTK_STATUSBAR(statusbar1), 0, "Cannot Perform Undo ");
 	if(max_undos_supported_count <= 0){
    	gtk_widget_set_sensitive(undo_button,FALSE);
 	}
 	
-	int k=0;
 
     if(no_of_valid_undos!=0){
 		
@@ -398,9 +458,9 @@ void on_undo_activate(GtkWidget *button,gpointer user_data)
         }
         else{
 		gint x_pos,y_pos;
-		x_pos = vacant_pos%4; 
-		y_pos = (int)(vacant_pos/4);
-		for(k=0;k<15;k++){
+		x_pos = vacant_pos%CELLPOS; 
+		y_pos = (int)(vacant_pos/CELLPOS);
+		for(k=0;k<NOOFBUTTONS;k++){
 			if((gint) gtk_object_get_data (GTK_OBJECT(buttons[k]), "current_pos") == (undo_store_positions->attach_in_vac_pos))
 				break;
 	}
@@ -433,7 +493,8 @@ void on_undo_activate(GtkWidget *button,gpointer user_data)
 		}else {
 			gtk_statusbar_push (GTK_STATUSBAR(statusbar1), 0, g_strdup_printf("Move No.: %i", move_no));
 		}
-	temp_undo_store_positions = undo_store_positions;
+
+	struct pos *temp_undo_store_positions = undo_store_positions;
        	undo_store_positions = undo_store_positions->link;
 	free(temp_undo_store_positions);
 	}	
@@ -450,7 +511,7 @@ void on_about1_activate (GtkMenuItem *menuitem, gpointer user_data)
                                         GTK_DIALOG_MODAL,
                                         GTK_MESSAGE_INFO,
                                         GTK_BUTTONS_OK,
-							"<b>Mpuzzle</b>\n\nSimple puzzle game\nHow to Play?\nClick on the number to move.\nIf you are able to sort numbers from 1 to 15\nthen you Win the Game!\nAuthor: Ramulu Kambalapuram,Ashok Chiruvella\nCompany: http://mars-india.com\nemail: ramulu.kambalapuram@mars-india.com");
+							"<b>Mpuzzle</b>\n\nSimple puzzle game\nHow to Play?\nClick on the number to move.\nIf you are able to sort numbers from 1 to 15\nthen you Win the Game!\nAuthor: Ashok Chiruvella, Ramulu Kambalapuram\nCompany: http://mars-india.com\nemail: ramulu.kambalapuram@mars-india.com");
 	
 	gtk_dialog_run (GTK_DIALOG (dialog));
 	gtk_widget_destroy (dialog);
