@@ -1,7 +1,31 @@
 #define IMAGES_DIR "/usr/share/pixmaps/"
 
+#define NOOFMOVES 50
+
 #define NO_OF_UNDOS_SUPPORTED 500
 #define MAXSIZE 256
+
+#define UP 0
+#define DOWN 1
+#define LEFT 2
+#define RIGHT 3
+
+#define NOOFBUTTONS 15
+#define NOOFPOSLIST 16
+#define BUF_SIZE 30
+#define ZERO 0
+#define THREE 3
+#define CELLPOS 4
+
+static int max_undos_supported_count=0;
+static int no_of_valid_undos=0;
+GtkWidget *undo_button;
+char buffer[BUF_SIZE];
+int alist[NOOFBUTTONS];
+int button_poslist[NOOFPOSLIST];
+int val_pos[4];
+int valid_moves_count=0;
+
 GtkWidget *main_window, *vbox1;
 GtkWidget *table1;
 GtkWidget *buttons[15];
@@ -12,6 +36,7 @@ GtkWidget *image;
 
 gint move_no;
 gint vacant_pos;
+gint vacant_prev_pos;
 
 struct pos {
         gint attach_in_vac_pos;	
@@ -21,31 +46,29 @@ struct pos {
 
 gboolean store_sequence_check_is_puzzle_solvable (void);
 
-void generate_puzzle (void);	
 
 GtkWidget* create_main_window (void);
 
-void swap_buttons (int p, int q);
+/* void swap_buttons (int p, int q);*/
 
 void on_new_activate (GtkWidget *button,GtkMenuItem *menuitem, gpointer user_data);
 
-void on_open1_activate (GtkMenuItem *menuitem, gpointer user_data);
-
-void on_save1_activate (GtkMenuItem *menuitem, gpointer user_data);
-
-void on_save_as1_activate (GtkMenuItem *menuitem, gpointer user_data);
-
 void on_quit1_activate (GtkMenuItem *menuitem, gpointer user_data);
 
-void on_cut1_activate (GtkMenuItem *menuitem, gpointer user_data);
-
-void on_copy1_activate (GtkMenuItem *menuitem, gpointer user_data);
-
-void on_paste1_activate (GtkMenuItem *menuitem, gpointer user_data);
-
-void on_delete1_activate (GtkMenuItem *menuitem, gpointer user_data);
-
 void on_about1_activate (GtkMenuItem *menuitem, gpointer user_data);
 
 void on_undo_activate (GtkWidget *button,gpointer user_data);
 
+void on_quit_confirm(GtkMenuItem *menuitem, gpointer user_data);
+
+void up();
+void down();
+void left();
+void right();
+gboolean is_move_correct(int x);
+void movement(int direction);
+
+void show_banner(void);		
+void store_index(void);
+
+void valid_move_positions(gint vcpos);
